@@ -1,4 +1,4 @@
-# Copyright 2014 Google Inc.
+# Copyright 2014 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -41,7 +41,7 @@ def _validate_name(name):
 
 
 class _PropertyMixin(object):
-    """Abstract mixin for cloud storage classes with associated propertties.
+    """Abstract mixin for cloud storage classes with associated properties.
 
     Non-abstract subclasses should implement:
       - client
