@@ -183,8 +183,8 @@ class Bucket(_PropertyMixin):
 
         :type chunk_size: int
         :param chunk_size: The size of a chunk of data whenever iterating
-                           (1 MB). This must be a multiple of 256 KB per the
-                           API specification.
+                           (in bytes). This must be a multiple of 256 KB per
+                           the API specification.
 
         :type encryption_key: bytes
         :param encryption_key:
@@ -1076,9 +1076,8 @@ class Bucket(_PropertyMixin):
     def requester_pays(self):
         """Does the requester pay for API requests for this bucket?
 
-        .. note::
-
-           No public docs exist yet for the "requester pays" feature.
+        See https://cloud.google.com/storage/docs/requester-pays for
+        details.
 
         :setter: Update whether requester pays for this bucket.
         :getter: Query whether requester pays for this bucket.
