@@ -1,4 +1,4 @@
-# Copyright 2014 Google Inc.
+# Copyright 2014 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -155,7 +155,7 @@ class Test_Bucket(unittest.TestCase):
 
     def test_bucket_name_value(self):
         BUCKET_NAME = 'bucket-name'
-        bucket = self._make_one(name=BUCKET_NAME)
+        self._make_one(name=BUCKET_NAME)
 
         bad_start_bucket_name = '/testing123'
         with self.assertRaises(ValueError):
@@ -239,6 +239,32 @@ class Test_Bucket(unittest.TestCase):
         with self.assertRaises(ValueError):
             bucket.create()
 
+    def test_create_w_missing_client_project(self):
+        BUCKET_NAME = 'bucket-name'
+        connection = _Connection()
+        client = _Client(connection, project=None)
+        bucket = self._make_one(client, BUCKET_NAME)
+
+        with self.assertRaises(ValueError):
+            bucket.create()
+
+    def test_create_w_explicit_project(self):
+        PROJECT = 'PROJECT'
+        BUCKET_NAME = 'bucket-name'
+        OTHER_PROJECT = 'other-project-123'
+        DATA = {'name': BUCKET_NAME}
+        connection = _Connection(DATA)
+        client = _Client(connection, project=PROJECT)
+        bucket = self._make_one(client, BUCKET_NAME)
+
+        bucket.create(project=OTHER_PROJECT)
+
+        kw, = connection._requested
+        self.assertEqual(kw['method'], 'POST')
+        self.assertEqual(kw['path'], '/b')
+        self.assertEqual(kw['query_params'], {'project': OTHER_PROJECT})
+        self.assertEqual(kw['data'], DATA)
+
     def test_create_hit(self):
         PROJECT = 'PROJECT'
         BUCKET_NAME = 'bucket-name'
@@ -473,7 +499,7 @@ class Test_Bucket(unittest.TestCase):
 
         self.assertEqual(len(notifications), len(resources))
         for notification, resource, topic_ref in zip(
-            notifications, resources, topic_refs):
+                notifications, resources, topic_refs):
             self.assertIsInstance(notification, BucketNotification)
             self.assertEqual(notification.topic_project, topic_ref[0])
             self.assertEqual(notification.topic_name, topic_ref[1])
@@ -701,6 +727,31 @@ class Test_Bucket(unittest.TestCase):
         self.assertEqual(kw['path'], COPY_PATH)
         self.assertEqual(kw['query_params'], {})
 
+    def test_copy_blobs_source_generation(self):
+        SOURCE = 'source'
+        DEST = 'dest'
+        BLOB_NAME = 'blob-name'
+        GENERATION = 1512565576797178
+
+        class _Blob(object):
+            name = BLOB_NAME
+            path = '/b/%s/o/%s' % (SOURCE, BLOB_NAME)
+
+        connection = _Connection({})
+        client = _Client(connection)
+        source = self._make_one(client=client, name=SOURCE)
+        dest = self._make_one(client=client, name=DEST)
+        blob = _Blob()
+        new_blob = source.copy_blob(blob, dest, source_generation=GENERATION)
+        self.assertIs(new_blob.bucket, dest)
+        self.assertEqual(new_blob.name, BLOB_NAME)
+        kw, = connection._requested
+        COPY_PATH = '/b/%s/o/%s/copyTo/b/%s/o/%s' % (SOURCE, BLOB_NAME,
+                                                     DEST, BLOB_NAME)
+        self.assertEqual(kw['method'], 'POST')
+        self.assertEqual(kw['path'], COPY_PATH)
+        self.assertEqual(kw['query_params'], {'sourceGeneration': GENERATION})
+
     def test_copy_blobs_preserve_acl(self):
         from google.cloud.storage.acl import ObjectACL
 
@@ -1266,8 +1317,8 @@ class Test_Bucket(unittest.TestCase):
         self.assertEqual(len(sent['bindings']), len(BINDINGS))
         key = operator.itemgetter('role')
         for found, expected in zip(
-            sorted(sent['bindings'], key=key),
-            sorted(BINDINGS, key=key)):
+                sorted(sent['bindings'], key=key),
+                sorted(BINDINGS, key=key)):
             self.assertEqual(found['role'], expected['role'])
             self.assertEqual(
                 sorted(found['members']), sorted(expected['members']))
@@ -1325,8 +1376,8 @@ class Test_Bucket(unittest.TestCase):
         self.assertEqual(len(sent['bindings']), len(BINDINGS))
         key = operator.itemgetter('role')
         for found, expected in zip(
-            sorted(sent['bindings'], key=key),
-            sorted(BINDINGS, key=key)):
+                sorted(sent['bindings'], key=key),
+                sorted(BINDINGS, key=key)):
             self.assertEqual(found['role'], expected['role'])
             self.assertEqual(
                 sorted(found['members']), sorted(expected['members']))
@@ -1539,7 +1590,7 @@ class Test_Bucket(unittest.TestCase):
         self.assertRaises(ValueError, bucket.make_public, recursive=True)
 
     def test_page_empty_response(self):
-        from google.api.core import page_iterator
+        from google.api_core import page_iterator
 
         connection = _Connection()
         client = _Client(connection)
